AnyIncDecCpCmp

Struct AnyIncDecCpCmp 

pub struct AnyIncDecCpCmp<T>
where T: PartialOrd + Clone + Copy,
{ /* private fields */ }
The Generic Implementation of IncDecCpCmp.

Acts as the general proxy layer for working with any type inside crate.

Note: unlike NumberIncDecCpCmp, the self.inc(a,b) and self.dec(a,b) are unchecked! If you are working with primitive numbers use: NumberIncDecCpCmp in stead.

Implementations

Source

impl<T> AnyIncDecCpCmp<T>
where T: PartialOrd + Clone + Copy,

pub fn new(min: T, max: T) -> Self

pub fn set_min(&mut self, v: T)

Sets the values returned by self.min() and self.min_ref(). Changing this value from the default will further constrain what ranges are considered invalid.

pub fn set_max(&mut self, v: T)

Sets the values returned by self.max() and self.max_ref(). Changing this value from the default will further constrain what ranges are considered invalid.