pub trait DefaultValues<T, V>: IncDecCpCmp<T, V> {
// Required methods
fn default_step(&self) -> V;
fn default_rebound(&self) -> V;
fn default_min() -> T;
fn default_max() -> T;
}Expand description
Default values
Implemenations of this trait drive the internals used for crate::iter::Intersector.
Required Methods
Sourcefn default_step(&self) -> V
fn default_step(&self) -> V
Returns the default value use for progressing a begin or end value of a range.
Sourcefn default_rebound(&self) -> V
fn default_rebound(&self) -> V
Returns the value used to adjust a start or end value in the context of std::ops::Bound.
Sourcefn default_min() -> T
fn default_min() -> T
Returns the default minimum value.
Sourcefn default_max() -> T
fn default_max() -> T
Returns the default maximum value.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".