OverlapIter

Struct OverlapIter 

pub struct OverlapIter<T, V, C: IncDecCpCmp<T, V>, R: GetBeginEnd<T>, F: GetBeginEndOption<T, R>> { /* private fields */ }
The core Iterator+DoubleEndedIterator for finding range intersections.

Implementations

Source

impl<T, V, C: IncDecCpCmp<T, V>, R: GetBeginEnd<T>, F: GetBeginEndOption<T, R>> OverlapIter<T, V, C, R, F>

pub fn new(src: Vec<R>, step: V, cmp: C, factory: F) -> Self

Creates a new OverlapIter from the slice of R.

Examples found in repository?
examples/getbeginend.rs (lines 38-47)
37fn main() {
38    for r in OverlapIter::new(
39        vec![
40            MyRange { a: 1, b: 4 },
41            MyRange { a: 3, b: 5 },
42            MyRange { a: 4, b: 6 },
43        ],
44        1,
45        NumberIncDecCpCmp::defaults(),
46        MyFactory {},
47    ) {
48        println!("{:?}", r);
49    }
50}
pub fn copy_range<U: GetBeginEnd<T>>(&self, src: &U) -> Option<R>

Tries to copy the src ref range via the internals. Returns None if it fails.

pub fn ln(&self) -> (Option<&R>, Option<&R>)

Returns a tuple of Option refs to the internal next and last_next ranges.

pub fn lb(&self) -> (Option<&R>, Option<&R>)

Returns a tuple of Option refs to the internal back and last_back ranges.

pub fn recompute_next(&mut self) -> Option<R>

After calling OverlapIter::update_column, call this method in place of OverlapIter::next to return the properly updated next value. Calling this method resets the curent state of OverlapIter::next_back.

pub fn recompute_back(&mut self) -> Option<R>

After calling OverlapIter::update_column, call this method in place of OverlapIter::next_back to return the properly updated back value. Calling this method resets the curent state of OverlapIter::next.

pub fn update_column( &mut self, idx: usize, range: R, ) -> Result<(), &'static str>

Updates the internal column to the new GetBeginEnd instance. Returns Result::Err if the range is invalid or if the index point does not exist.

After calling this method the next call to OverlapIter::next or OverlapIter::next_back will not function correctly. First call the respective OverlapIter::recompute_next or OverlapIter::recompute_back, then proceed to call either OverlapIter::next or OverlapIter::next_back normally.

pub fn try_next(&self, src: &Option<R>) -> Option<R>

Genrates a new next based on the src passed in.

pub fn try_next_back(&self, src: &Option<R>) -> Option<R>

Genrates a new back based on the src passed in.