RangeRelation

Enum RangeRelation 

pub enum RangeRelation<T> {
    Before(T),
    Overlap(T),
    After(T),
    Last(T),
    Invalid(T),
}
This enum is used to represent positional relationships between 2 ranges.

The additional states, represent the initialization of the set.

Variants



Before(T)

Range a is before range b



Overlap(T)

Range a and b overlap



After(T)

Range a is after range b



Last(T)

Represents final set of data



Invalid(T)

Denotes a was not valid

Implementations

Source

impl<T> RangeRelation<T>

pub fn unwrap(self) -> T

Unwraps the Value.

pub fn invalid(self) -> T

Unwraps the state of RangeRelation::Invalid or panics!.

pub fn last(self) -> T

Unwraps the state of RangeRelation::Last or panics!

pub fn before(self) -> T

Unwraps the state of RangeRelation::Before or panics!

pub fn overlap(self) -> T

Unwraps the state of RangeRelation::Overlap or panics!

pub fn after(self) -> T

Unwraps the state of RangeRelation::After or panics!

pub fn is_last(&self) -> bool

Returns true if RangeRelation::Last.

pub fn is_invalid(&self) -> bool

Returns true if RangeRelation::Invalid.

pub fn is_overlap(&self) -> bool

Returns true if RangeRelation::Overlap.

pub fn is_before(&self) -> bool

Returns true if RangeRelation::Before.

pub fn is_after(&self) -> bool

Returns true if RangeRelation::After.