pub fn first_range_begin_end<T, V, C: IncDecCpCmp<T, V>, R: GetBeginEnd<T>>(
src: &[R],
step: &V,
t: &C,
) -> Option<(T, T)>Expand description
Looks for the first most range, if found returns an Option<(T,T)>.
pub fn first_range_begin_end<T, V, C: IncDecCpCmp<T, V>, R: GetBeginEnd<T>>(
src: &[R],
step: &V,
t: &C,
) -> Option<(T, T)>Looks for the first most range, if found returns an Option<(T,T)>.