next_range_begin_end

Function next_range_begin_end 

pub fn next_range_begin_end<T, V, C: IncDecCpCmp<T, V>, R: GetBeginEnd<T>>(
    begin: &T,
    src: &[R],
    step: &V,
    t: &C,
) -> Option<(T, T)>
Searches for the next smallest range valid range of (T,T) overlaps with begin. If no range overlaps with end, it finds the next smallest range after begin. Returns None when no matches were found.