previous_range_begin_end

Function previous_range_begin_end 

pub fn previous_range_begin_end<T, V, C: IncDecCpCmp<T, V>, R: GetBeginEnd<T>>(
    end: &T,
    src: &[R],
    step: &V,
    t: &C,
) -> Option<(T, T)>
Searches for the previous smallest range valid range of (T,T) overlaps with end. If no range overlaps with end, it finds the previous smallest range before begin. Returns None when no matches were found.