pub fn retool_begin<T, V, C: IncDecCpCmp<T, V>, R: GetBeginEnd<T>>(
res: (T, T),
src: &[R],
step: &V,
t: &C,
) -> (T, T)Expand description
Computes the final value from a result of crate::reduce_back.
pub fn retool_begin<T, V, C: IncDecCpCmp<T, V>, R: GetBeginEnd<T>>(
res: (T, T),
src: &[R],
step: &V,
t: &C,
) -> (T, T)Computes the final value from a result of crate::reduce_back.