The crate::utils module represents the static methods that drive most of the overlap and consolidation intenrals. The static methods are exposed for general use and testing.
Enums
- Range
Relation - This enum is used to represent positional relationships between 2 ranges.
Functions
- consolidate
- This function is the stateless implementation of crate::Consolidate.
- first_
range_ begin_ end - Looks for the first most range, if found returns an Option<(T,T)>.
- grow
- Given 2 instances of GetBeginEnd it returns a range that contains both.
- last_
range_ begin_ end - Looks for the last most range, if found returns an Option<(T,T)>.
- min_max
- Returns an Option wrapped tuple representing the smallest begin and largest end values found in src.
- next_
range_ begin_ end - Searches for the next smallest range valid range of (T,T) overlaps with begin. If no range overlaps with end, it finds the next smallest range after begin. Returns None when no matches were found.
- next_
smallest_ range - previous_
range_ begin_ end - Searches for the previous smallest range valid range of (T,T) overlaps with end. If no range overlaps with end, it finds the previous smallest range before begin. Returns None when no matches were found.
- previous_
smallest_ range - Given the current begin and end, returns the smallest next range from src.
- range_
bounds_ to_ values - Range to value Conversion
- range_
relation - Compares the positional relationship between a and b.
- reduce_
back - Given the current begin and end, returns the smallest back range from src.
- reduce_
next - retool_
begin - Computes the final value from a result of crate::reduce_back.
- retool_
end - Computes the final value from a result of crate::reduce_next.
- sort_
forward - Compares range a and b and returns the Forward Consolidation Order std::cmp::Ordering value.
- sort_
reverse - Compares range a and b and returns the Reverse Consolidation Order std::cmp::Ordering value.