pub fn sort_reverse<T, V, R: RangeBounds<T>, S: RangeBounds<T>, C: IncDecCpCmp<T, V>>(
x: &R,
y: &S,
rebound: &V,
t: &C,
) -> OrderingExpand description
Compares range a and b and returns the Reverse Consolidation Order std::cmp::Ordering value.
The sort order is meant to represent Reverse Consolidation Order not tradtional range sort order. Reverse Consolidation Order is represented as latest largest ranges first.
Put another way:
- GetBeginEnd.get_end() desc
- GetBeginEnd.get_begin() asc
Panics!
If the RangeBounds cannot be converted to a computable range!